/**
 * The React App file
 */

// React libraries
import React, { Fragment, useEffect, useRef, useState } from 'react';

// Stylings
import './App.scss';

// Application components
import {
    BryntumSchedulerPro,

} from '@bryntum/schedulerpro-react';
import {
    RecurringTimeSpan,
    RecurringTimeSpansMixin,
    ResourceTimeRangeModel,
    ResourceTimeRangeStore, Store, TimeSpan
  } from '@bryntum/schedulerpro';
import { schedulerConfig } from './AppConfig';
import dayjs from 'dayjs';
import localizedFormat from 'dayjs/plugin/localizedFormat';
import customParseFormat from 'dayjs/plugin/customParseFormat';
import localeData from 'dayjs/plugin/localeData';
import utc from 'dayjs/plugin/utc';
import ParentEvent  from './ParentEvent';

export const UTC_DATE_FORMAT = 'YYYY-MM-DD HH:mm:ss.SSS';
//
export const FIRST_INDEX = 0;

dayjs.extend(localizedFormat);
dayjs.extend(customParseFormat);
dayjs.extend(localeData);
dayjs.extend(utc);

const data={
    resources: [
        {
          "calendar": "O175U3DEXD-9CN7U3DBOI",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "6PM to 11PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "JE9EP6EN0G"
          ],
          "timezoneOffset": -330,
          "workGroups": "AshCrewTaskType1",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "cmCrewPartialAccess"
            ],
            "isAllFilterApplied": false
          },
          "name": "Ashu tech2",
          "id": "O175U3DEXD",
          "workforceCalendarId": "9CN7U3DBOI"
        },
        {
          "calendar": "0XS5U3DD7K-3SZVMDETYT",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "24 hour shift",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "3DPKT3DIV",
            "7O1KT3DLS7",
            "7YIJT3DO38",
            "QMPHT3DVV2"
          ],
          "timezoneOffset": -330,
          "workGroups": "AshuWorkOrderType1WG1,AshuTaskType1WG1,AshuRegion1WG1,AshuCust1WG1",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU"
            ],
            "workOrderType": [
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "TK1JT3DCEK"
            ],
            "customer": [
              "2SZ37NBGS3"
            ]
          },
          "name": "Ashu tech3",
          "id": "0XS5U3DD7K",
          "workforceCalendarId": "3SZVMDETYT"
        },
        {
          "calendar": "NPURX3DGP0-TG067NB5EV",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "9AM to 5PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "3DPKT3DIV",
            "7O1KT3DLS7",
            "7YIJT3DO38",
            "QMPHT3DVV2"
          ],
          "timezoneOffset": -330,
          "workGroups": "AshuWorkOrderType1WG1,AshuTaskType1WG1,AshuRegion1WG1,AshuCust1WG1",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU"
            ],
            "workOrderType": [
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "TK1JT3DCEK"
            ],
            "customer": [
              "2SZ37NBGS3"
            ]
          },
          "name": "Ashu tech4",
          "id": "NPURX3DGP0",
          "workforceCalendarId": "TG067NB5EV"
        },
        {
          "calendar": "E2MSX3DXC7-TG067NB5EV",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "9AM to 5PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "3DPKT3DIV",
            "7O1KT3DLS7",
            "7YIJT3DO38",
            "QMPHT3DVV2"
          ],
          "timezoneOffset": -330,
          "workGroups": "AshuWorkOrderType1WG1,AshuTaskType1WG1,AshuRegion1WG1,AshuCust1WG1",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU"
            ],
            "workOrderType": [
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "TK1JT3DCEK"
            ],
            "customer": [
              "2SZ37NBGS3"
            ]
          },
          "name": "Ashu tech5",
          "id": "E2MSX3DXC7",
          "workforceCalendarId": "TG067NB5EV"
        },
        {
          "calendar": "JM6TX3DIL1-3SZVMDETYT",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "24 hour shift",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "3DPKT3DIV",
            "7O1KT3DLS7",
            "7YIJT3DO38",
            "JE9EP6EN0G",
            "QMPHT3DVV2"
          ],
          "timezoneOffset": -330,
          "workGroups": "AshuWorkOrderType1WG1,AshuTaskType1WG1,AshuRegion1WG1,AshCrewTaskType1,AshuCust1WG1",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU",
              "cmCrewPartialAccess"
            ],
            "workOrderType": [
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "TK1JT3DCEK"
            ],
            "customer": [
              "2SZ37NBGS3"
            ]
          },
          "name": "Ashu tech6",
          "id": "JM6TX3DIL1",
          "workforceCalendarId": "3SZVMDETYT"
        },
        {
          "calendar": "5HQ4ULBKH1-TG067NB5EV",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "9AM to 5PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "3DPKT3DIV",
            "7O1KT3DLS7",
            "7YIJT3DO38",
            "QMPHT3DVV2"
          ],
          "timezoneOffset": -330,
          "workGroups": "AshuWorkOrderType1WG1,AshuTaskType1WG1,AshuRegion1WG1,AshuCust1WG1",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU"
            ],
            "workOrderType": [
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "TK1JT3DCEK"
            ],
            "customer": [
              "2SZ37NBGS3"
            ]
          },
          "name": "Ashutosh",
          "id": "5HQ4ULBKH1",
          "workforceCalendarId": "TG067NB5EV"
        },
        {
          "calendar": "8PPT9TA5C4-CM4FH7C2ZS",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "GENERAL SHIFT",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "Skill2 - L2,Repair & Maintenance - Default,Skill1 - L1,Beginner - Repair - L1",
          "important": 1,
          "workGroupIdsList": [
            "C1KII7CZPK"
          ],
          "timezoneOffset": -330,
          "workGroups": "WG1-R1",
          "userWorkGroupFilterIdData": {
            "isAllFilterApplied": false,
            "region": [
              "1GG67NB3VJ"
            ]
          },
          "name": "Raj Trivedi",
          "id": "8PPT9TA5C4",
          "workforceCalendarId": "CM4FH7C2ZS"
        },
        {
          "calendar": "5L11CTAH9E-GHAC3VA2SZ",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "Default Shift 10AM - 6PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "Skill2 - L2,Repair & Maintenance - Default,Expert- Repair - L1",
          "important": 1,
          "workGroupIdsList": [
            "GY5OECET3U",
            "HCRE9EEFNO",
            "HYHNECE3SD",
            "LDA6AEEK1A"
          ],
          "timezoneOffset": -330,
          "workGroups": "Bengaluru Tasks All,Bengaluru Cust All,Bengaluru Regions All,Bengaluru WO All",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU",
              "preventiveMaintenance",
              "YQALZ592PE"
            ],
            "workOrderType": [
              "4O9NZ59M6Z",
              "HFY1NTBPYW",
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "2OMLECEAMK",
              "TK1JT3DCEK",
              "ZVVLECEO96"
            ],
            "customer": [
              "2C06N8CT4A",
              "FEQ90EEK53"
            ]
          },
          "name": "Saurav",
          "id": "5L11CTAH9E",
          "workforceCalendarId": "GHAC3VA2SZ"
        },
        {
          "calendar": "LG59ECE5YT-9CN7U3DBOI",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "6PM to 11PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "GY5OECET3U",
            "HCRE9EEFNO",
            "HYHNECE3SD",
            "LDA6AEEK1A"
          ],
          "timezoneOffset": -330,
          "workGroups": "Bengaluru Tasks All,Bengaluru Cust All,Bengaluru Regions All,Bengaluru WO All",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU",
              "preventiveMaintenance",
              "YQALZ592PE"
            ],
            "workOrderType": [
              "4O9NZ59M6Z",
              "HFY1NTBPYW",
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "2OMLECEAMK",
              "TK1JT3DCEK",
              "ZVVLECEO96"
            ],
            "customer": [
              "2C06N8CT4A",
              "FEQ90EEK53"
            ]
          },
          "name": "Saurav OA",
          "id": "LG59ECE5YT",
          "workforceCalendarId": "9CN7U3DBOI"
        },
        {
          "calendar": "KYF6ECEIAN-TG067NB5EV",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "9AM to 5PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "GY5OECET3U",
            "HCRE9EEFNO",
            "HYHNECE3SD",
            "LDA6AEEK1A"
          ],
          "timezoneOffset": -330,
          "workGroups": "Bengaluru Tasks All,Bengaluru Cust All,Bengaluru Regions All,Bengaluru WO All",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU",
              "preventiveMaintenance",
              "YQALZ592PE"
            ],
            "workOrderType": [
              "4O9NZ59M6Z",
              "HFY1NTBPYW",
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "2OMLECEAMK",
              "TK1JT3DCEK",
              "ZVVLECEO96"
            ],
            "customer": [
              "2C06N8CT4A",
              "FEQ90EEK53"
            ]
          },
          "name": "Saurav PA",
          "id": "KYF6ECEIAN",
          "workforceCalendarId": "TG067NB5EV"
        },
        {
          "calendar": "51Q9ECELBN-GHAC3VA2SZ",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "Default Shift 10AM - 6PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "GY5OECET3U",
            "HCRE9EEFNO",
            "HYHNECE3SD",
            "LDA6AEEK1A"
          ],
          "timezoneOffset": -330,
          "workGroups": "Bengaluru Tasks All,Bengaluru Cust All,Bengaluru Regions All,Bengaluru WO All",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU",
              "preventiveMaintenance",
              "YQALZ592PE"
            ],
            "workOrderType": [
              "4O9NZ59M6Z",
              "HFY1NTBPYW",
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "2OMLECEAMK",
              "TK1JT3DCEK",
              "ZVVLECEO96"
            ],
            "customer": [
              "2C06N8CT4A",
              "FEQ90EEK53"
            ]
          },
          "name": "Saurav PU",
          "id": "51Q9ECELBN",
          "workforceCalendarId": "GHAC3VA2SZ"
        },
        {
          "calendar": "GSP4ECEP9E-9CN7U3DBOI",
          "image": "https://d122pusslfmuhu.cloudfront.net/UserNoProfilePic.jpg",
          "shift": "6PM to 11PM",
          "icons": [
            "@iconlib.WifiOff"
          ],
          "skills": "-",
          "important": 1,
          "workGroupIdsList": [
            "GY5OECET3U",
            "HCRE9EEFNO",
            "HYHNECE3SD",
            "LDA6AEEK1A"
          ],
          "timezoneOffset": -330,
          "workGroups": "Bengaluru Tasks All,Bengaluru Cust All,Bengaluru Regions All,Bengaluru WO All",
          "userWorkGroupFilterIdData": {
            "taskType": [
              "L4SW9O8XJU",
              "preventiveMaintenance",
              "YQALZ592PE"
            ],
            "workOrderType": [
              "4O9NZ59M6Z",
              "HFY1NTBPYW",
              "VPHX9O8BRF"
            ],
            "isAllFilterApplied": false,
            "region": [
              "2OMLECEAMK",
              "TK1JT3DCEK",
              "ZVVLECEO96"
            ],
            "customer": [
              "2C06N8CT4A",
              "FEQ90EEK53"
            ]
          },
          "name": "Saurav SD",
          "id": "GSP4ECEP9E",
          "workforceCalendarId": "9CN7U3DBOI"
        }
      ],
      events:[
        {
          "workOrderTaskAppointmentWindowStartDate": "2023-03-01 10:30:00.000",
          "minStartDate": "2023-03-01 10:30:00.000",
          "maxEndDate": "2023-03-01 12:30:00.000",
          "resourceId": "51Q9ECELBN",
          "workOrderTypeId": "VPHX9O8BRF",
          "endDate": "2023-03-01 17:30:00.000",
          "statusColor": "#0aa9c2",
          "customerCompanyName": "Saurav as a customer",
          "regionName": "Bellandur-Bengaluru",
          "siteName": "Saurav's new home site",
          "type": "Corrective Maintenance - Single Technician",
          "taskAtRisk": 0,
          "taskTypeName": "Corrective Maintenance - Single Technician",
          "tooltipHeader": "<b>Corrective Maintenance - Single Technician</b><br/>TSK000053",
          "duration": 60,
          "taskTypeId": "L4SW9O8XJU",
          "scheduledStartTime": "2023-03-01 16:30:00.000",
          "customerId": "2C06N8CT4A",
          "workOrderTaskId": "TSK000053",
          "workOrderTaskEstimatedDuration": 1,
          "id": "9O13VPE5ST-51Q9ECELBN",
          "multiBreakdownTaskFlag": false,
          "durationUnit": "minute",
          "workOrderTaskEstimatedDurationMinutes": 60,
          "workOrderTaskAppointmentWindowEndDate": "2023-03-01 12:30:00.000",
          "priorityIndex": "10",
          "alloweEventOverlap": false,
          "workOrderTaskAppointmentBasedFlag": 1,
          "calendar": "2023-3-1 16:00:00 - 2023-3-1 18:00:00",
          "workOrderTaskAppointmentWindow": "[{\"start\":\"2023-03-01 10:30:00.000\",\"end\":\"2023-03-01 12:30:00.000\"}]",
          "workOrderTaskAppointmentWindowId": "NIE86PE7AB",
          "scheduledEndTime": "2023-03-01 17:30:00",
          "resultWebPageId": "GEZKCPA4BA",
          "icons": "",
          "taskTypeCrewTask": 0,
          "crewTask": 0,
          "workOrderTaskTypeId": "L4SW9O8XJU",
          "statusId": "workOrderTaskScheduled",
          "regionId": "ZVVLECEO96",
          "eventColor": "#0aa9c2",
          "name": "[High] TSK000053",
          "workOrderTaskScheduledDate": "2023-03-01 16:30:00.000",
          "workOrderRequestedDate": "2022-12-29 09:14:03.486",
          "startDate": "2023-03-01 16:30:00.000",
          "status": "Scheduled"
        }
      ],
      calenders: [
        {
          "intervals": [
            {
              "endDate": "2023-03-01 12:30:00.000",
              "name": "",
              "startDate": "2023-03-01 10:30:00.000",
              "isWorking": true
            }
          ],
          "name": "Time Slot",
          "id": "2023-3-1 16:00:00 - 2023-3-1 18:00:00",
          "unspecifiedTimeIsWorking": false
        }
      ]
}
class ResourceTimeRange extends RecurringTimeSpan(ResourceTimeRangeModel) {}

export class ResourceTimeRangesStore extends
  RecurringTimeSpansMixin(ResourceTimeRangeStore) {
  static get defaultConfig () {
    return {
      // use our new MyResourceTimeRange model
      modelClass: ResourceTimeRange
    };
  }
}

class TimeRange extends RecurringTimeSpan(TimeSpan) {}
export class TimeRangeStore extends RecurringTimeSpansMixin(Store) {
    static get defaultConfig () {
      return {
        // use our new MyTimeRange model
        modelClass: TimeRange,
        storeId: 'timeRanges'
      };
    }
  }


const App = () => {
    const scheduler = useRef();
    const resourceTimeRangeStore = new ResourceTimeRangesStore();

    const timeRangeStores = new TimeRangeStore();
    const [refreshPage, setRefreshPage] = useState(false);
    const getSchedulerInstance = () => scheduler?.current?.instance;
    const highlightDeliveryWindow = (eventRecord) => {
        const { minStartDate, maxEndDate } = eventRecord;
        const scheduler = getSchedulerInstance();
        if (scheduler && minStartDate && maxEndDate) {
          scheduler.highlightTimeSpan({
            // Optional, to support animations
            animationId: 'deliveryWindow',
            // Highlight surrounding area
            surround: true,
            name: `OUTSIDE OF APPOINTMENT WINDOW`,
            // The time span to visualize
            startDate: minStartDate,
            endDate: maxEndDate
          });
        }
      };
      useEffect(()=>{
        refreshAction();
      },[]);
    // Timeline size change handler


    const refreshAction = ()=>{
        
        setRefreshPage(!refreshPage);
    }


  const unHighlighTimeSpan = () => {

      const scheduler = getSchedulerInstance();
      scheduler.unhighlightTimeSpans(true);

  };
console.log('render....');
    return (
        <Fragment>
            {/* BryntumDemoHeader component is used for Bryntum example styling only and can be removed */}

            <div id="content">
                <BryntumSchedulerPro
                    {...schedulerConfig}

       project={{
            resourcesData: data.resources,
            eventsData: data.events,
            timeRangeStore: timeRangeStores,
            timeRangesData: [],
            resourceTimeRangeStore: resourceTimeRangeStore,
            calendarsData: data.calenders,
            resourceTimeRangesData:[],
            eventModelClass: ParentEvent
          }}
                    ref={scheduler}
                    listeners ={{
                        scheduleClick() {
                            unHighlighTimeSpan();
                        },
                        eventDragStart({ eventRecords }) {
                              const selectedEvent = eventRecords[FIRST_INDEX];
                              const highlightObj={
                                minStartDate: selectedEvent.startDate,
                                maxEndDate: selectedEvent.endDate
                              };
                              highlightDeliveryWindow(highlightObj);
                            
                        },
                        eventDragReset(source) {
                            unHighlighTimeSpan();
                        },
                        eventResizeStart({ eventRecord }) {
                            highlightDeliveryWindow(eventRecord);
                        },
                        eventResizeEnd() {
                            unHighlighTimeSpan();
                        }
                    }}
                    eventDragFeature={{
                        validatorFn ({ startDate }) {
                          if (dayjs().isAfter(startDate, 'second')) {
                            return {
                              valid: false,
                              message: "past date"
                            };
                          }
                          return {
                            valid: true
                          };
                        }
                      }}
                      resourceTimeRangesFeature={true}
                      resourceNonWorkingTimeFeature={true}
                      nonWorkingTimeFeature={true}
                />
            </div>
        </Fragment>
    );
};

export default App;
